<?php
namespace Misuzu;

use RuntimeException;

if(!$msz->isLoggedIn() || !perms_check_user(MSZ_PERMS_USER, $msz->getActiveUser()->getId(), MSZ_PERM_USER_MANAGE_WARNINGS)) {
    echo render_error(403);
    return;
}

$users = $msz->getUsers();

$userInfos = [
    $msz->getActiveUser()->getId() => $msz->getActiveUser(),
];
$userColours = [
    $msz->getActiveUser()->getId() => $users->getUserColour($msz->getActiveUser()),
];

$filterUser = null;
if(filter_has_var(INPUT_GET, 'u')) {
    $filterUserId = filter_input(INPUT_GET, 'u', FILTER_SANITIZE_NUMBER_INT);
    try {
        $filterUser = $users->getUser($filterUserId, 'id');
        $userInfos[$filterUserId] = $filterUser;
        $userColours[$filterUserId] = $users->getUserColour($filterUser);
    } catch(RuntimeException $ex) {
        echo render_error(404);
        return;
    }
}

$warns = $msz->getWarnings();
$pagination = new Pagination($warns->countWarnings(userInfo: $filterUser), 10);

if(!$pagination->hasValidOffset()) {
    echo render_error(404);
    return;
}

$warnList = [];
$warnInfos = $warns->getWarnings(userInfo: $filterUser, pagination: $pagination);

foreach($warnInfos as $warnInfo) {
    if(array_key_exists($warnInfo->getUserId(), $userInfos))
        $userInfo = $userInfos[$warnInfo->getUserId()];
    else
        $userInfos[$warnInfo->getUserId()] = $userInfo = $users->getUser($warnInfo->getUserId(), 'id');

    if(array_key_exists($userInfo->getId(), $userColours))
        $userColour = $userColours[$userInfo->getId()];
    else
        $userColours[$userInfo->getId()] = $userColour = $users->getUserColour($userInfo);

    if(!$warnInfo->hasModId()) {
        $modInfo = null;
        $modColour = null;
    } else {
        if(array_key_exists($warnInfo->getModId(), $userInfos))
            $modInfo = $userInfos[$warnInfo->getModId()];
        else
            $userInfos[$warnInfo->getModId()] = $modInfo = $users->getUser($warnInfo->getModId(), 'id');

        if(array_key_exists($modInfo->getId(), $userColours))
            $modColour = $userColours[$modInfo->getId()];
        else
            $userColours[$modInfo->getId()] = $modColour = $users->getUserColour($modInfo);
    }

    $warnList[] = [
        'info' => $warnInfo,
        'user' => $userInfo,
        'user_colour' => $userColour,
        'mod' => $modInfo,
        'mod_colour' => $modColour,
    ];
}

Template::render('manage.users.warnings', [
    'manage_warns' => $warnList,
    'manage_warns_pagination' => $pagination,
    'manage_warns_filter_user' => $filterUser,
]);
